 

import java.net.*;
import java.io.*;

/**
 * Serverimplementierung, Auslagerung der Clientprozesse in einen Thread<br/>
 * Der Zugriff auf die Ressource plaetzevorhanden ist <b>nicht synchronisiert</b>!
 * Zur Simulation wird ein Platzkontigent immer wieder aufgef&uuml;llt.
 * @author ISB-Arbeitskreis, Umsetzungshilfen Informatik 12
 * @version 1.0
 */
public class SERVER5 {
    /** bidirektionale Schnittstelle zur Netzwerkprotokoll-Implementierung des Servers*/
    private ServerSocket serverSocket = null;
    /** Portnummer des Ports, auf dem die Verbindungen etabliert sind*/
    private int port;
   /**speichert die aktuelle Anzahl der noch vorhandenen Pl&auml;tze*/
    private int plaetzevorhanden = 3;
    /**dieses Platzkontignent wird immer wieder aufgef&uuml;llt*/
    private static int platzkontingent = 10;
    /**Beim Abbruch der Simulation durch &Uuml;berbuchung wird Tastatureingabe erwartet*/
    private BufferedReader tastatur;

    /**
     * Konstruktor des Servers
     * @exception IOException eine Ausnahme tritt auf falls:<br/>
     * - der Server nicht gestartet werden kann (weil beispielsweise der Port nicht frei ist)<br/>
     * - die Clientverbindung gest&ouml;rt bzw. unterbrochen wurde.
     */
   public SERVER5() throws IOException {

        ServerStarten();
        while (true) {//wartet immer auf neue Clientverbindungen
            AufNeuenClientWarten();
        }
    }

   /**
     * wartet auf eine neue Verbindung und lagert diese einen Thread aus
     */
    private void AufNeuenClientWarten() throws IOException {
        System.out.println("warte auf Client, hoere auf Port " + port);
        Socket clientSocket = serverSocket.accept();  //warten auf die Verbindung
        CLIENTPROZESS2 clientprozess = new CLIENTPROZESS2(clientSocket, this) ;
        clientprozess.start();
        System.out.println("Clientprozess gestartet...");
    }

    /**
     * gibt die Anzahl der noch verf&uuml;gbaren Pl&auml;tze zur&uuml;ck
     * @return Anzahl der verf&uuml;gbaren Pl&auml;tze
     */
     public int PlaetzeVerfuegbar() {
        return plaetzevorhanden;
    }

    /**die angegebene Anzahl an Pl&auml;tzen buchen
     * @param anzahl, die Anzahl der zu buchenden Pl&auml;tze
     * @return gibt false zur&uuml;ck, falls die Buchung aus Platzmangel nicht
     * durchgef&uuml;hrt werden konnte
     */
   public boolean PlaetzeBuchen(int anzahl) {

        boolean plaetzebuchbar = (anzahl <= plaetzevorhanden);

        try {// das Ermitteln, ob Plaetze buchbar sind brauche etwas Zeit ...
            Thread.sleep((int) (Math.random() * 100));
        } catch (InterruptedException e) {
            System.err.println("Fehler in der Serververabeitung.");
        }

        if (plaetzebuchbar) {
            plaetzevorhanden = plaetzevorhanden - anzahl;
            System.out.println(plaetzevorhanden + " Plaetze vorhanden.");

            if (plaetzevorhanden < 0) {
                System.out.println("\u00DCberbuchung ! Beenden mit Return...");
                try {
                    tastatur.read();
                    System.exit(0);
                } catch (Exception e) {
                    System.err.println("Fehler in der Serververabeitung.");
                }

            }
            if (plaetzevorhanden == 0) {
                plaetzevorhanden = platzkontingent;
            }
            return true;
        } else {
            return false;
        }
    }

    /**
     * fragt den Port ab und erzeugt den Serversocket
     */
    private void ServerStarten() throws IOException {

        System.out.println("Port eingeben: ");
        tastatur = new BufferedReader(new InputStreamReader(System.in));
        //Server Socket erzeugen
        port = Integer.parseInt(tastatur.readLine());
        serverSocket = new ServerSocket(port);
        System.out.println("Server gestartet...");
    }

    /**
     * Hauptprogramm zum Erzeugen des Serverobjekts
     * @param args keine Parameter beim Programmaufruf erforderlich
     */
    public static void main(String[] args) throws IOException {
        try {
            new SERVER5();
        } catch (Exception e) {
            System.err.println("Fehler in der Serververabeitung.");
            System.exit(1);
        }
    }
}